@@ -1,4 +1,4 @@
-from typing import List, Generator, Union
+from typing import List, Generator, Union, Iterable, Optional
 
 import torch
 from esm.pretrained import load_model_and_alphabet_local
@@ -11,8 +11,9 @@ class ESMEmbedderBase(EmbedderInterface):
     # The only thing we need to overwrite is the name and _picked_layer
     embedding_dimension = 1280
     number_of_layers = 1  # Following ESM, we only consider layer 34 (ESM) or 33 (ESM1b)
-
     necessary_files = ["model_file"]
+    # https://github.com/facebookresearch/esm/issues/49#issuecomment-803110092
+    max_len = 1022
 
     _picked_layer: int
 
@@ -30,6 +31,7 @@ class ESMEmbedderBase(EmbedderInterface):
 
     def embed_batch(self, batch: List[str]) -> Generator[ndarray, None, None]:
         """https://github.com/facebookresearch/esm/blob/dfa524df54f91ef45b3919a00aaa9c33f3356085/README.md#quick-start-"""
+        self._assert_max_len(batch)
         data = [(str(pos), sequence) for pos, sequence in enumerate(batch)]
         batch_labels, batch_strs, batch_tokens = self._batch_converter(data)
 
@@ -44,6 +46,20 @@ class ESMEmbedderBase(EmbedderInterface):
         for i, (_, seq) in enumerate(data):
             yield token_embeddings[i, 1 : len(seq) + 1].cpu().numpy()
 
+    def embed_many(
+        self, sequences: Iterable[str], batch_size: Optional[int] = None
+    ) -> Generator[ndarray, None, None]:
+        self._assert_max_len(sequences)
+        return super().embed_many(sequences, batch_size)
+
+    def _assert_max_len(self, sequences: Iterable[str]):
+        max_len = max(len(i) for i in sequences)
+        if max_len > self.max_len:
+            raise ValueError(
+                f"{self.name} only allows sequences up to {self.max_len} residues, "
+                f"but your longest sequence is {max_len} residues long"
+            )
+
     @staticmethod
     def reduce_per_protein(embedding: ndarray) -> ndarray:
         return embedding.mean(0)
