@@ -1,4 +1,6 @@
 import logging
+import numpy
+
 import torch
 import collections
 
@@ -54,7 +56,7 @@ _disor_labels = {
 }
 
 BasicSecondaryStructureResult = collections.namedtuple('BasicSecondaryStructureResult', 'DSSP3 DSSP8 disorder')
-BasicSubcellularLocalizationResult = collections.namedtuple('BasicSubcellularLocalizationResult', 'localization membrane')
+SubcellularLocalizationAndMembraneBoundness = collections.namedtuple('SubcellularLocalizationAndMembraneBoundness', 'localization membrane')
 BasicExtractedAnnotations = collections.namedtuple('BasicExtractedAnnotations', 'DSSP3 DSSP8 disorder localization membrane')
 
 
@@ -76,7 +78,7 @@ class BasicAnnotationExtractor(object):
         # Create un-trained (raw) model and ensure self._model_type is valid
         if self._model_type == "seqvec_from_publication":
             self._subcellular_location_model = SUBCELL_FNN().to(self._device)
-        elif self._model_type == "bert_from_publication": # Drop batchNorm for ProtTrans models
+        elif self._model_type == "bert_from_publication" or self._model_type == "t5_xl_u50_from_publication": # Drop batchNorm for ProtTrans models
             self._subcellular_location_model = SUBCELL_FNN(use_batch_norm=False).to(self._device)
         else:
             print("You first need to define your custom model architecture.")
@@ -91,7 +93,6 @@ class BasicAnnotationExtractor(object):
         self._subcellular_location_checkpoint_file = self._options['subcellular_location_checkpoint_file']
 
         # Read in pre-trained model
-
         self._secondary_structure_model = SECSTRUCT_CNN().to(self._device)
 
         # load pre-trained weights for annotation machines
@@ -106,7 +107,8 @@ class BasicAnnotationExtractor(object):
         self._subcellular_location_model.eval()
         self._secondary_structure_model.eval()
 
-    def get_subcellular_location(self, raw_embedding: ndarray) -> BasicSubcellularLocalizationResult:
+    def get_subcellular_location(self, raw_embedding: ndarray) -> SubcellularLocalizationAndMembraneBoundness:
+        raw_embedding = raw_embedding.astype(numpy.float32)  # For T5 fp16
         # Reduce embedding to fixed size, per-sequence (aka: Lx3x2014 --> 1024).
         # This is similar to embedder.reduce_per_protein(),
         # but more efficient since may be run in GPU (see self._device)
@@ -120,8 +122,8 @@ class BasicAnnotationExtractor(object):
         if self._model_type == "seqvec_from_publication":
             # SeqVec case
             embedding = torch.tensor(raw_embedding).to(self._device).sum(dim=0).mean(dim=0, keepdim=True)
-        elif self._model_type == "bert_from_publication":
-            # Bert case
+        elif self._model_type == "bert_from_publication" or self._model_type == "t5_xl_u50_from_publication":
+            # Bert/T5 case
             embedding = torch.tensor(raw_embedding).to(self._device).mean(dim=0, keepdim=True)
         else:
             raise NotImplementedError
@@ -131,16 +133,16 @@ class BasicAnnotationExtractor(object):
         pred_loc = _loc_labels[torch.max(yhat_loc, dim=1)[1].item()]  # get index of output node with max. activation,
         pred_mem = _mem_labels[torch.max(yhat_mem, dim=1)[1].item()]  # this corresponds to the predicted class
 
-        return BasicSubcellularLocalizationResult(localization=pred_loc, membrane=pred_mem)
+        return SubcellularLocalizationAndMembraneBoundness(localization=pred_loc, membrane=pred_mem)
 
     def get_secondary_structure(self, raw_embedding: ndarray) -> BasicSecondaryStructureResult:
-        # TODO: xxmh: same as for subcell loc.:
-        #       SeqVec requires summing over layers while ProtTrans models only extract last layers
+        raw_embedding = raw_embedding.astype(numpy.float32)  # For T5 fp16
+        # same as for subcell loc.: SeqVec requires summing over layers while ProtTrans models only extract last layers
         if self._model_type == "seqvec_from_publication":
             # SeqVec case
             embedding = torch.tensor(raw_embedding).to(self._device).sum(dim=0, keepdim=True).permute(0, 2, 1).unsqueeze(dim=-1)
-        elif self._model_type == "bert_from_publication":
-            # Bert case
+        elif self._model_type == "bert_from_publication" or self._model_type == "t5_xl_u50_from_publication":
+            # Bert/T5 case
             # Flip dimensions for ProtTrans models in order to make feature dimension the first dimension
             embedding = torch.tensor(raw_embedding).to(self._device).T[None, :, :, None]
         else:
