@@ -18,12 +18,14 @@ __all__ = [
     "FileManagerInterface",
     "FileSystemFileManager",
     "InvalidParameterError",
+    "QueryEmbeddingsFile",
     "check_required",
     "get_device",
     "get_file_manager",
     "get_model_directories_from_zip",
     "get_model_file",
     "read_fasta",
+    "read_mapping_file",
     "reindex_h5_file",
     "reindex_sequences",
     "write_fasta_file",
