@@ -1,16 +1,18 @@
-#!/bin/python
+#!/usr/bin/env python3
 
 import argparse
 import logging
 
-from bio_embeddings.utilities.pipeline import run
+from bio_embeddings.utilities.pipeline import parse_config_file_and_execute_run
 
 
 def main():
     """
     Pipeline commandline entry point
     """
-    logging.basicConfig(level=logging.INFO, format="%(levelname)s %(message)s")
+    logging.basicConfig(level=logging.INFO, format="%(asctime)s %(levelname)s %(message)s")
+    # Jax likes to print warnings
+    logging.captureWarnings(True)
 
     parser = argparse.ArgumentParser(description='Embeds protein sequences.')
     parser.add_argument('-o', '--overwrite', dest='overwrite', required=False, action='store_true',
@@ -19,7 +21,7 @@ def main():
                         help='The path to the config. For examples, see folder "parameter examples".')
     arguments = parser.parse_args()
 
-    run(arguments.config_path[0], overwrite=arguments.overwrite)
+    parse_config_file_and_execute_run(arguments.config_path[0], overwrite=arguments.overwrite)
 
 
 if __name__ == '__main__':
