@@ -1,7 +1,11 @@
 import abc
 
 
-class FileManagerInterface(object, metaclass=abc.ABCMeta):
+class FileManagerInterface(metaclass=abc.ABCMeta):
+
+    @abc.abstractmethod
+    def __init__(self):
+        pass
 
     @abc.abstractmethod
     def create_prefix(self, prefix) -> str:
