@@ -183,6 +183,15 @@ class QueryEmbeddingsFile:
     either an MD5 hash of the input sequence, or an integer (if `remapping_simple: True`).
 
     Available for embeddings created with the pipeline starting with v0.1.5
+
+    .. code-block:: python
+
+        import h5py
+        from bio_embeddings.utilities import QueryEmbeddingsFile
+
+        with h5py.File("path/to/file.h5", "r") as file:
+            embedding_querier = QueryEmbeddingsFile(file)
+            print(embedding_querier.query_original_id("Some_Database_ID_1234").mean())
     """
 
     def __init__(self, embeddings_file: h5py.File):
@@ -212,3 +221,12 @@ class QueryEmbeddingsFile:
         :return: the embedding as a numpy array
         """
         return np.array(self._embeddings_file[new_id])
+
+
+def read_mapping_file(mapping_file: str) -> DataFrame:
+    """Reads mapping_file.csv and ensures consistent types"""
+    # We want to read the unnamed column 0 as str (esp. with simple_remapping), which requires some workarounds
+    # https://stackoverflow.com/a/29793294/3549270
+    mapping_file = read_csv(mapping_file, index_col=0)
+    mapping_file.index = mapping_file.index.astype("str")
+    return mapping_file
